@@ -529,7 +529,7 @@ static int readheaders(int sock,
 	    sizeticker += linelen;
 	    while (sizeticker >= SIZETICKER)
 	    {
-		if (!run.use_syslog)
+		if (!run.use_syslog && isatty(1))
 		{
 		    fputc('.', stdout);
 		    fflush(stdout);
@@ -725,6 +725,7 @@ static int readheaders(int sock,
 	else if (!strncasecmp("Resent-Sender:", line, 14))
 	    resent_sender_offs = (line - msgblk.headers);
 
+#ifdef __UNUSED__
  	else if (!strncasecmp("Message-Id:", buf, 11))
 	{
 	    if (ctl->server.uidl)
@@ -740,6 +741,7 @@ static int readheaders(int sock,
 		}
  	    }
  	}
+#endif /* __UNUSED__ */
 
 	else if (!MULTIDROP(ctl))
 	    continue;
@@ -1045,7 +1047,7 @@ static int readheaders(int sock,
 	free_str_list(&msgblk.recipients);
 	return(PS_IOERR);
     }
-    else if (!run.use_syslog && outlevel >= O_VERBOSE)
+    else if ((run.poll_interval == 0 || nodetach) && outlevel >= O_VERBOSE && isatty(2))
 	fputs("#", stderr);
 
     /* write error notifications */
@@ -1155,7 +1157,7 @@ static int readbody(int sock, struct query *ctl, flag forward, int len)
 	    sizeticker += linelen;
 	    while (sizeticker >= SIZETICKER)
 	    {
-		if (!run.use_syslog && outlevel > O_SILENT)
+		if ((run.poll_interval == 0 || nodetach) && outlevel > O_SILENT && isatty(1))
 		{
 		    fputc('.', stdout);
 		    fflush(stdout);
@@ -1213,7 +1215,7 @@ static int readbody(int sock, struct query *ctl, flag forward, int len)
 		release_sink(ctl);
 		return(PS_IOERR);
 	    }
-	    else if (outlevel >= O_VERBOSE)
+	    else if (outlevel >= O_VERBOSE && isatty(1))
 	    {
 		fputc('*', stdout);
 		fflush(stdout);
@@ -1415,7 +1417,7 @@ static void send_size_warnings(struct query *ctl)
 	    nbr = current->val.status.mark;
 	    size = atoi(current->id);
 	    stuff_warning(ctl, 
-		    _("\t%d msg %d octets long skipped by fetchmail.\n"),
+		    _("\t%d msg %d octets long skipped by fetchmail.\r\n"),
 		    nbr, size);
 	}
 	current->val.status.num++;
@@ -1482,8 +1484,9 @@ const int maxfetch;		/* maximum number of messages to fetch */
 	if (js == THROW_SIGPIPE)
 	{
 	    report(stdout,
-		   _("SIGPIPE thrown from an MDA or a stream socket error"));
+		   _("SIGPIPE thrown from an MDA or a stream socket error\n"));
 	    ok = PS_SOCKET;
+	    goto cleanUp;
 	}
 	else if (js == THROW_TIMEOUT)
 	{
@@ -1502,7 +1505,7 @@ const int maxfetch;		/* maximum number of messages to fetch */
 		       ctl->mda ? "MDA" : "SMTP");
 	    else if (phase == LISTENER_WAIT)
 		report(stdout,
-		       _("timeout after %d seconds waiting for listener to respond.\n"));
+		       _("timeout after %d seconds waiting for listener to respond.\n"), ctl->server.timeout);
 	    else
 		report(stdout, 
 		       _("timeout after %d seconds.\n"), ctl->server.timeout);
@@ -1537,13 +1540,13 @@ const int maxfetch;		/* maximum number of messages to fetch */
 	/* try to clean up all streams */
 	release_sink(ctl);
 	if (ctl->smtp_socket != -1)
-	    close(ctl->smtp_socket);
+	    SockClose(ctl->smtp_socket);
 	if (mailserver_socket != -1)
 	    SockClose(mailserver_socket);
     }
     else
     {
-	char buf[POPBUFSIZE+1], *realhost;
+	char buf[MSGBUFSIZE+1], *realhost;
 	int len, num, count, new, bytes, deletions = 0, *msgsizes = NULL;
 #if INET6_ENABLE
 	int fetches, dispatches, oldphase;
@@ -1585,6 +1588,7 @@ const int maxfetch;		/* maximum number of messages to fetch */
 	if ((mailserver_socket = SockOpen(realhost, port, NULL, ctl->server.plugin)) == -1)
 #endif /* INET6_ENABLE */
 	{
+	    char	errbuf[BUFSIZ];
 #if !INET6_ENABLE
 	    int err_no = errno;
 #ifdef HAVE_RES_SEARCH
@@ -1596,30 +1600,48 @@ const int maxfetch;		/* maximum number of messages to fetch */
 	     * in daemon mode but the connection to the outside world
 	     * is down.
 	     */
-	    if (err_no == EHOSTUNREACH && run.poll_interval)
-	        goto ehostunreach;
-
-	    report_build(stderr, _("fetchmail: %s connection to %s failed"), 
-			 protocol->name, ctl->server.pollname);
-#ifdef HAVE_RES_SEARCH
-	    if (h_errno != 0)
+	    if (!((err_no == EHOSTUNREACH || err_no == ENETUNREACH) 
+		  && run.poll_interval))
 	    {
-		if (h_errno == HOST_NOT_FOUND)
-		    report_complete(stderr, _(": host is unknown\n"));
-		else if (h_errno == NO_ADDRESS)
-		    report_complete(stderr, _(": name is valid but has no IP address\n"));
-		else if (h_errno == NO_RECOVERY)
-		    report_complete(stderr, _(": unrecoverable name server error\n"));
-		else if (h_errno == TRY_AGAIN)
-		    report_complete(stderr, _(": temporary name server error\n"));
+		report_build(stderr, _("fetchmail: %s connection to %s failed"), 
+			     protocol->name, ctl->server.pollname);
+#ifdef HAVE_RES_SEARCH
+		if (h_errno != 0)
+		{
+		    if (h_errno == HOST_NOT_FOUND)
+			strcpy(errbuf, _("host is unknown."));
+		    else if (h_errno == NO_ADDRESS)
+			strcpy(errbuf, _("name is valid but has no IP address."));
+		    else if (h_errno == NO_RECOVERY)
+			strcpy(errbuf, _("unrecoverable name server error."));
+		    else if (h_errno == TRY_AGAIN)
+			strcpy(errbuf, _("temporary name server error."));
+		    else
+			sprintf(errbuf, _("unknown DNS error %d."), h_errno);
+		}
 		else
-		    report_complete(stderr, _(": unknown DNS error %d\n"), h_errno);
-	    }
-	    else
 #endif /* HAVE_RES_SEARCH */
-		report_complete(stderr, ": %s\n", strerror(err_no));
+		    strcpy(errbuf, strerror(err_no));
+		report_complete(stderr, ": %s\n", errbuf);
 
-	ehostunreach:
+#ifdef __UNUSED
+		/* 
+		 * Don't use this.  It was an attempt to address Debian bug
+		 * #47143 (Notify user by mail when pop server nonexistent).
+		 * Trouble is, that doesn't work; you trip over the case 
+		 * where your SLIP or PPP link is down...
+		 */
+		/* warn the system administrator */
+		if (open_warning_by_mail(ctl, (struct msgblk *)NULL) == 0)
+		{
+#define OPENFAIL	"Subject: Fetchmail unreachable-server warning.\r\n\r\nFetchmail could not reach the mail server %s:"
+		    stuff_warning(ctl, OPENFAIL, ctl->server.pollname);
+		    stuff_warning(ctl, errbuf, ctl->server.pollname);
+		    close_warning_by_mail(ctl, (struct msgblk *)NULL);
+#undef OPENFAIL
+		}
+#endif
+	    }
 #endif /* INET6_ENABLE */
 	    ok = PS_SOCKET;
 	    set_timeout(0);
@@ -2002,7 +2024,7 @@ const int maxfetch;		/* maximum number of messages to fetch */
 			     */
 			    if (protocol->fetch_body && !suppress_readbody) 
 			    {
-				if (outlevel >= O_VERBOSE)
+				if (outlevel >= O_VERBOSE && isatty(1))
 				{
 				    fputc('\n', stdout);
 				    fflush(stdout);
@@ -2056,7 +2078,7 @@ const int maxfetch;		/* maximum number of messages to fetch */
 				/* tell server we got it OK and resynchronize */
 				if (protocol->trail)
 				{
-				    if (outlevel >= O_VERBOSE)
+				    if (outlevel >= O_VERBOSE && isatty(1))
 				    {
 					fputc('\n', stdout);
 					fflush(stdout);
