@@ -81,7 +81,7 @@ void envquery(int argc, char **argv)
 	program_name = argv[0];
 
 #define RCFILE_NAME	".fetchmailrc"
-    rcfile = (char *) xmalloc(strlen(home)+strlen(RCFILE_NAME)+2);
+    rcfile = (char *) xmalloc(strlen(home)+sizeof(RCFILE_NAME)+1);
     /* avoid //.fetchmailrc */
     if (strcmp(home, "/") != 0) {
     	strcpy(rcfile, home);
@@ -221,6 +221,11 @@ char *visbuf(const char *buf)
 	    *tp++ = '\\'; *tp++ = '"';
 	    buf++;
 	}
+	else if (*buf == '\\')
+	{
+	    *tp++ = '\\'; *tp++ = '\\';
+	    buf++;
+	}
 	else if (isprint(*buf) || *buf == ' ')
 	    *tp++ = *buf++;
 	else if (*buf == '\n')
