@@ -117,11 +117,13 @@ static int etrn_logout(int sock, struct query *ctl)
 const static struct method etrn =
 {
     "ETRN",		/* ESMTP ETRN extension */
-#if INET6
+#if INET6_ENABLE
     "smtp",		/* standard SMTP port */
-#else /* INET6 */
+    "smtps",		/* ssl SMTP port */
+#else /* INET6_ENABLE */
     25,			/* standard SMTP port */
-#endif /* INET6 */
+    465,			/* ssl SMTP port */
+#endif /* INET6_ENABLE */
     FALSE,		/* this is not a tagged protocol */
     FALSE,		/* this does not use a message delimiter */
     etrn_ok,		/* parse command response */
