@@ -6,7 +6,6 @@
 #include "config.h"
 
 #include <stdio.h>
-#include <ctype.h>
 #if defined(STDC_HEADERS)
 #include <stdlib.h>
 #endif
@@ -16,7 +15,6 @@
 #include <fcntl.h>
 #include <string.h>
 #include <signal.h>
-#include <getopt.h>
 #if defined(HAVE_SYSLOG)
 #include <syslog.h>
 #endif
@@ -25,15 +23,11 @@
 #include <grp.h>
 #endif
 #include <errno.h>
-#include <sys/time.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #ifdef HAVE_SETRLIMIT
 #include <sys/resource.h>
 #endif /* HAVE_SETRLIMIT */
-#ifdef HAVE_SYS_WAIT_H
-#include <sys/wait.h>
-#endif /* HAVE_SYS_WAIT_H */
 #include <sys/utsname.h>
 
 #ifdef HAVE_GETHOSTBYNAME
@@ -73,7 +67,6 @@ char *home;		    /* invoking user's home directory */
 char *program_name;	    /* the name to prefix error messages with */
 flag configdump;	    /* dump control blocks for configurator */
 const char *fetchmailhost;  /* either `localhost' or the host's FQDN */
-volatile int lastsig;		/* last signal received */
 
 #if NET_SECURITY
 void *request = NULL;
@@ -90,34 +83,6 @@ static time_t parsetime;	/* time of last parse */
 static void terminate_run(int);
 static void terminate_poll(int);
 
-#ifdef SLEEP_WITH_ALARM
-/*
- * The function of this variable is to remove the window during which a
- * SIGALRM can hose the code (ALARM is triggered *before* pause() is called).
- * This is a bit of a kluge; the real right thing would use sigprocmask(),
- * sigsuspend().  This workaround lets the interval timer trigger the first
- * alarm after the required interval and will then generate alarms all 5
- * seconds, until it is certain, that the critical section (ie., the window)
- * is left.
- */
-#if defined(STDC_HEADERS)
-static sig_atomic_t	alarm_latch = FALSE;
-#else
-/* assume int can be written in one atomic operation on non ANSI-C systems */
-static int		alarm_latch = FALSE;
-#endif
-
-RETSIGTYPE gotsigalrm(sig)
-int sig;
-{
-    signal(sig, gotsigalrm);
-    lastsig = sig;
-    alarm_latch = TRUE;
-}
-#endif /* SLEEP_WITH_ALARM */
-
-RETSIGTYPE donothing(int sig) {signal(sig, donothing); lastsig = sig;}
-
 #ifdef HAVE_ON_EXIT
 static void unlockit(int n, void *p)
 #else
@@ -129,19 +94,6 @@ static void unlockit(void)
 	unlink(lockfile);
 }
 
-#ifdef __EMX__
-/* Various EMX-specific definitions */
-int itimerflag;
-void itimerthread(void* dummy) {
-  if (outlevel >= O_VERBOSE)
-    report(stderr, _("fetchmail: thread sleeping for %d sec.\n"), poll_interval);
-  while(1) {
-    _sleep2(poll_interval*1000);
-    kill((getpid()), SIGALRM);
-  }
-}
-#endif
-
 #ifdef __FreeBSD__
 /* drop SGID kmem privileage until we need it */
 static void dropprivs(void)
@@ -335,11 +287,12 @@ int main(int argc, char **argv)
 	if (ctl->active && !(implicitmode && ctl->server.skip)&&!ctl->password)
 	{
 	    if (ctl->server.preauthenticate == A_KERBEROS_V4 ||
-		ctl->server.preauthenticate == A_KERBEROS_V5 ||
+			ctl->server.preauthenticate == A_KERBEROS_V5 ||
+			ctl->server.preauthenticate == A_SSH ||
 #ifdef GSSAPI
-		ctl->server.protocol == P_IMAP_GSS ||
+			ctl->server.protocol == P_IMAP_GSS ||
 #endif /* GSSAPI */
-		ctl->server.protocol == P_IMAP_K4)
+			ctl->server.protocol == P_IMAP_K4)
 		/* Server won't care what the password is, but there
 		   must be some non-null string here.  */
 		ctl->password = ctl->remotename;
@@ -509,22 +462,30 @@ int main(int argc, char **argv)
     for (ctl = querylist; ctl; ctl = ctl->next)
     {
 	if (ctl->active && !(implicitmode && ctl->server.skip)
-	    && ctl->server.protocol != P_ETRN 
-	    && ctl->server.protocol != P_IMAP_K4
+		&& ctl->server.protocol != P_ETRN 
+		&& ctl->server.protocol != P_IMAP_K4
 #ifdef GSSAPI
-	    && ctl->server.protocol != P_IMAP_GSS
+		&& ctl->server.protocol != P_IMAP_GSS
 #endif /* GSSAPI */
-	    && !ctl->password)
-	{
-	    char* password_prompt = _("Enter password for %s@%s: ");
+		&& !ctl->password)
+	    if (!isatty(0))
+	    {
+		fprintf(stderr,
+			_("fetchmail: can't find a password for %s@s.\n"),
+			ctl->remotename, ctl->server.pollname);
+		return(PS_AUTHFAIL);
+	    }
+	    else
+	    {
+		char* password_prompt = _("Enter password for %s@%s: ");
 
-	    xalloca(tmpbuf, char *, strlen(password_prompt) +
-		    strlen(ctl->remotename) +
-		    strlen(ctl->server.pollname) + 1);
-	    (void) sprintf(tmpbuf, password_prompt,
-			   ctl->remotename, ctl->server.pollname);
-	    ctl->password = xstrdup((char *)getpassword(tmpbuf));
-	}
+		xalloca(tmpbuf, char *, strlen(password_prompt) +
+			strlen(ctl->remotename) +
+			strlen(ctl->server.pollname) + 1);
+		(void) sprintf(tmpbuf, password_prompt,
+			       ctl->remotename, ctl->server.pollname);
+		ctl->password = xstrdup((char *)getpassword(tmpbuf));
+	    }
     }
 
     /*
@@ -574,7 +535,11 @@ int main(int argc, char **argv)
     signal(SIGQUIT, terminate_run);
 
     /* here's the exclusion lock */
-    if ((st = open(lockfile, O_WRONLY | O_CREAT | O_EXCL, 0666)) != -1) {
+#ifndef O_SYNC
+#define O_SYNC	0	/* use it if we have it */
+#endif
+    if ((st = open(lockfile, O_WRONLY|O_CREAT|O_EXCL|O_SYNC, 0666)) != -1)
+    {
 	sprintf(tmpbuf,"%d", getpid());
 	write(st, tmpbuf, strlen(tmpbuf));
 	if (run.poll_interval)
@@ -582,7 +547,7 @@ int main(int argc, char **argv)
 	    sprintf(tmpbuf," %d", run.poll_interval);
 	    write(st, tmpbuf, strlen(tmpbuf));
 	}
-	close(st);	/* should be safe, previous write was atomic */
+	close(st);	/* should be safe, fd was opened with O_SYNC */
 	lock_acquired = TRUE;
     }
 
@@ -601,7 +566,12 @@ int main(int argc, char **argv)
 	struct stat	rcstat;
 
 	if (stat(rcfile, &rcstat) == -1)
-	    report(stderr, _("couldn't time-check %s\n"), rcfile);
+	{
+	    if (errno != ENOENT)
+		report(stderr, 
+		       _("couldn't time-check %s (error %d)\n"),
+		       rcfile, errno);
+	}
 	else if (rcstat.st_mtime > parsetime)
 	{
 	    report(stdout, _("restarting fetchmail (%s changed)\n"), rcfile);
@@ -666,7 +636,39 @@ int main(int argc, char **argv)
 		    successes++;
 		else if (!check_only && 
 			 ((querystatus!=PS_NOMAIL) || (outlevel==O_DEBUG)))
-		    report(stdout, _("Query status=%d\n"), querystatus);
+		switch(querystatus)
+		{
+		case PS_SUCCESS:
+		    report(stdout, "Query status=0 (SUCCESS)\n"); break ;
+		case PS_NOMAIL: 
+		    report(stdout, "Query status=1 (NOMAIL)\n"); break ;
+		case PS_SOCKET:
+		    report(stdout, "Query status=2 (SOCKET)\n"); break ;
+		case PS_AUTHFAIL:
+		    report(stdout, "Query status=3 (AUTHFAIL)\n"); break ;
+		case PS_PROTOCOL:
+		    report(stdout, "Query status=4 (PROTOCOL)\n"); break ;
+		case PS_SYNTAX:
+		    report(stdout, "Query status=5 (SYNTAX)\n"); break ;
+		case PS_IOERR:
+		    report(stdout, "Query status=6 (IOERR)\n"); break ;
+		case PS_ERROR:
+		    report(stdout, "Query status=7 (ERROR)\n"); break ;
+		case PS_EXCLUDE:
+		    report(stdout, "Query status=8 (EXCLUDE)\n"); break ;
+		case PS_LOCKBUSY:
+		    report(stdout, "Query status=9 (LOCKBUSY)\n"); break ;
+		case PS_SMTP:
+		    report(stdout, "Query status=10 (SMTP)\n"); break ;
+		case PS_DNS:
+		    report(stdout, "Query status=11 (DNS)\n"); break ;
+		case PS_BSMTP:
+		    report(stdout, "Query status=12 (BSMTP)\n"); break ;
+		case PS_MAXFETCH:
+		    report(stdout, "Query status=13 (MAXFETCH)\n"); break ;
+		default:
+		    report(stdout, _("Query status=%d\n"), querystatus); break;
+		}
 
 #if (defined(linux) && !INET6_ENABLE) || defined (__FreeBSD__)
 		if (ctl->server.monitor)
@@ -695,6 +697,8 @@ int main(int argc, char **argv)
 	 */
 	if (run.poll_interval)
 	{
+	    int lastsig;
+
 	    /* 
 	     * Because passwords can expire, it may happen that *all*
 	     * hosts are now out of the loop due to authfail
@@ -720,118 +724,24 @@ int main(int argc, char **argv)
 		       _("fetchmail: sleeping at %s\n"), rfc822timestamp());
 
 	    /*
-	     * With this simple hack, we make it possible for a foreground 
-	     * fetchmail to wake up one in daemon mode.  What we want is the
-	     * side effect of interrupting any sleep that may be going on,
-	     * forcing fetchmail to re-poll its hosts.  The second line is
-	     * for people who think all system daemons wake up on SIGHUP.
+	     * OK, now pause util it's time for the next poll cycle.
+	     * A TRUE return indicates we received a wakeup signal;
+	     * unwedge all servers in case the problem has been
+	     * manually repaired.
 	     */
-	    signal(SIGUSR1, donothing);
-	    if (!getuid())
-		signal(SIGHUP, donothing);
-
-	    /* time for a pause in the action... */
+	    if ((lastsig = idle(run.poll_interval)))
 	    {
-#ifndef __EMX__
-#ifdef SLEEP_WITH_ALARM		/* not normally on */
-		/*
-		 * We can't use sleep(3) here because we need an alarm(3)
-		 * equivalent in order to implement server nonresponse timeout.
-		 * We'll just assume setitimer(2) is available since fetchmail
-		 * has to have a BSDoid socket layer to work at all.
-		 */
-		/* 
-		 * This code stopped working under glibc-2, apparently due
-		 * to the change in signal(2) semantics.  (The siginterrupt
-		 * line, added later, should fix this problem.) John Stracke
-		 * <francis@netscape.com> wrote:
-		 *
-		 * The problem seems to be that, after hitting the interval
-		 * timer while talking to the server, the process no longer
-		 * responds to SIGALRM.  I put in printf()s to see when it
-		 * reached the pause() for the poll interval, and I checked
-		 * the return from setitimer(), and everything seemed to be
-		 * working fine, except that the pause() just ignored SIGALRM.
-		 * I thought maybe the itimer wasn't being fired, so I hit
-		 * it with a SIGALRM from the command line, and it ignored
-		 * that, too.  SIGUSR1 woke it up just fine, and it proceeded
-		 * to repoll--but, when the dummy server didn't respond, it
-		 * never timed out, and SIGALRM wouldn't make it.
-		 *
-		 * (continued below...)
-		 */
-		struct itimerval ntimeout;
-
-		ntimeout.it_interval.tv_sec = 5; /* repeat alarm every 5 secs */
-		ntimeout.it_interval.tv_usec = 0;
-		ntimeout.it_value.tv_sec  = run.poll_interval;
-		ntimeout.it_value.tv_usec = 0;
-
-		siginterrupt(SIGALRM, 1);
-		alarm_latch = FALSE;
-		signal(SIGALRM, gotsigalrm);	/* first trap signals */
-		setitimer(ITIMER_REAL,&ntimeout,NULL);	/* then start timer */
-		/* there is a very small window between the next two lines */
-		/* which could result in a deadlock.  But this will now be  */
-		/* caught by periodical alarms (see it_interval) */
-		if (!alarm_latch)
-		    pause();
-		/* stop timer */
-		ntimeout.it_interval.tv_sec = ntimeout.it_interval.tv_usec = 0;
-		ntimeout.it_value.tv_sec  = ntimeout.it_value.tv_usec = 0;
-		setitimer(ITIMER_REAL,&ntimeout,NULL);	/* now stop timer */
-		signal(SIGALRM, SIG_IGN);
-#else
-		/* 
-		 * So the workaround I used is to make it sleep by using
-		 * select() instead of setitimer()/pause().  select() is
-		 * perfectly happy being called with a timeout and
-		 * no file descriptors; it just sleeps until it hits the
-		 * timeout.  The only concern I had was that it might
-		 * implement its timeout with SIGALRM--there are some
-		 * Unices where this is done, because select() is a library
-		 * function--but apparently not.
-		 */
-                struct timeval timeout;
-
-                timeout.tv_sec = run.poll_interval;
-                timeout.tv_usec = 0;
-                do {
-                    lastsig = 0;
-                    select(0,0,0,0, &timeout);
-                } while (lastsig == SIGCHLD);
-#endif
-#else /* EMX */
-		alarm_latch = FALSE;
-		signal(SIGALRM, gotsigalrm);
-		_beginthread(itimerthread, NULL, 32768, NULL);
-		/* see similar code above */
-		if (!alarm_latch)
-		    pause();
-		signal(SIGALRM, SIG_IGN);
-#endif /* ! EMX */
-		if (lastsig == SIGUSR1
-			|| ((run.poll_interval && !getuid()) && lastsig == SIGHUP))
-		{
 #ifdef SYS_SIGLIST_DECLARED
-		    report(stdout, 
-			   _("awakened by %s\n"), sys_siglist[lastsig]);
+		report(stdout, 
+		       _("awakened by %s\n"), sys_siglist[lastsig]);
 #else
-		    report(stdout, 
-			   _("awakened by signal %d\n"), lastsig);
+		report(stdout, 
+		       _("awakened by signal %d\n"), lastsig);
 #endif
-		    /* received a wakeup - unwedge all servers in case */
-		    /* the problem has been manually repaired          */
-		    for (ctl = querylist; ctl; ctl = ctl->next)
-		        ctl->wedged = FALSE;
-		}
+		for (ctl = querylist; ctl; ctl = ctl->next)
+		    ctl->wedged = FALSE;
 	    }
 
-	    /* now lock out interrupts again */
-	    signal(SIGUSR1, SIG_IGN);
-	    if (!getuid())
-		signal(SIGHUP, SIG_IGN);
-
 	    if (outlevel >= O_VERBOSE)
 		report(stdout, _("awakened at %s\n"), rfc822timestamp());
 	}
@@ -912,6 +822,7 @@ static void optmerge(struct query *h2, struct query *h1, int force)
     FLAG_MERGE(pass8bits);
     FLAG_MERGE(dropstatus);
     FLAG_MERGE(mimedecode);
+    FLAG_MERGE(idle);
     FLAG_MERGE(limit);
     FLAG_MERGE(warnings);
     FLAG_MERGE(fetchlimit);
@@ -953,10 +864,11 @@ static int load_params(int argc, char **argv, int optind)
     def_opts.listener = SMTP_MODE;
 
     /* note the parse time, so we can pick up on modifications */
-    if (stat(rcfile, &rcstat) == -1)
-	report(stderr, _("couldn't time-check the run-control file\n"));
-    else
+    parsetime = 0;	/* foil compiler warnings */
+    if (stat(rcfile, &rcstat) != -1)
 	parsetime = rcstat.st_mtime;
+    else if (errno != ENOENT)
+	report(stderr, _("couldn't time-check the run-control file\n"));
 
     /* this builds the host list */
     if ((st = prc_parse_file(rcfile, !versioninfo)) != 0)
@@ -1027,6 +939,27 @@ static int load_params(int argc, char **argv, int optind)
     /* use localhost if we never fetch the FQDN of this host */
     fetchmailhost = "localhost";
 
+    /* here's where we override globals */
+    if (cmd_run.logfile)
+	run.logfile = cmd_run.logfile;
+    if (cmd_run.idfile)
+	run.idfile = cmd_run.idfile;
+    /* do this before the keep/fetchall test below, otherwise -d0 may fail */
+    if (cmd_run.poll_interval >= 0)
+	run.poll_interval = cmd_run.poll_interval;
+    if (cmd_run.invisible)
+	run.invisible = cmd_run.invisible;
+    if (cmd_run.use_syslog)
+	run.use_syslog = (cmd_run.use_syslog == FLAG_TRUE);
+    if (cmd_run.postmaster)
+	run.postmaster = cmd_run.postmaster;
+    if (cmd_run.bouncemail)
+	run.bouncemail = cmd_run.bouncemail;
+
+    /* check and daemon options are not compatible */
+    if (check_only && run.poll_interval)
+	run.poll_interval = 0;
+
     /* merge in wired defaults, do sanity checks and prepare internal fields */
     for (ctl = querylist; ctl; ctl = ctl->next)
     {
@@ -1056,6 +989,7 @@ static int load_params(int argc, char **argv, int optind)
 	    DEFAULT(ctl->pass8bits, FALSE);
 	    DEFAULT(ctl->dropstatus, FALSE);
 	    DEFAULT(ctl->mimedecode, FALSE);
+	    DEFAULT(ctl->idle, FALSE);
 	    DEFAULT(ctl->server.dns, TRUE);
 	    DEFAULT(ctl->server.uidl, FALSE);
 #ifdef	SSL_ENABLE
@@ -1248,26 +1182,6 @@ static int load_params(int argc, char **argv, int optind)
 	}
     }
 
-    /* here's where we override globals */
-    if (cmd_run.logfile)
-	run.logfile = cmd_run.logfile;
-    if (cmd_run.idfile)
-	run.idfile = cmd_run.idfile;
-    if (cmd_run.poll_interval >= 0)
-	run.poll_interval = cmd_run.poll_interval;
-    if (cmd_run.invisible)
-	run.invisible = cmd_run.invisible;
-    if (cmd_run.use_syslog)
-	run.use_syslog = (cmd_run.use_syslog == FLAG_TRUE);
-    if (cmd_run.postmaster)
-	run.postmaster = cmd_run.postmaster;
-    if (cmd_run.bouncemail)
-	run.bouncemail = cmd_run.bouncemail;
-
-    /* check and daemon options are not compatible */
-    if (check_only && run.poll_interval)
-	run.poll_interval = 0;
-
 #ifdef POP3_ENABLE
     /* initialize UID handling */
     if (!versioninfo && (st = prc_filecheck(run.idfile, !versioninfo)) != 0)
@@ -1292,7 +1206,7 @@ static int load_params(int argc, char **argv, int optind)
 }
 
 static void terminate_poll(int sig)
-/* to be executed at the nd of a poll cycle */
+/* to be executed at the end of a poll cycle */
 {
     /*
      * Close all SMTP delivery sockets.  For optimum performance
@@ -1322,7 +1236,7 @@ static void terminate_poll(int sig)
 	    if (ctl->smtp_socket != -1)
 	    {
 		SMTP_quit(ctl->smtp_socket);
-		close(ctl->smtp_socket);
+		SockClose(ctl->smtp_socket);
 		ctl->smtp_socket = -1;
 	    }
     }
@@ -1533,7 +1447,7 @@ static void dump_params (struct runctl *runp,
 
 	if (ctl->server.protocol == P_POP3 
 #if INET6_ENABLE
-	    && !strcmp(ctl->server.service, KPOP_PORT)
+	    && ctl->server.service && !strcmp(ctl->server.service, KPOP_PORT)
 #else /* INET6_ENABLE */
 	    && ctl->server.port == KPOP_PORT
 #endif /* INET6_ENABLE */
@@ -1560,8 +1474,10 @@ static void dump_params (struct runctl *runp,
 	putchar('\n');
 	if (ctl->server.preauthenticate == A_KERBEROS_V4)
 	    printf(_("  Kerberos V4 preauthentication enabled.\n"));
-	if (ctl->server.preauthenticate == A_KERBEROS_V5)
+	else if (ctl->server.preauthenticate == A_KERBEROS_V5)
 	    printf(_("  Kerberos V5 preauthentication enabled.\n"));
+	else if (ctl->server.preauthenticate == A_SSH)
+	    printf(_("  End-to-end encryption assumed.\n"));
 #ifdef	SSL_ENABLE
 	if (ctl->use_ssl)
 	    printf("  SSL encrypted sessions enabled.\n");
@@ -1609,6 +1525,9 @@ static void dump_params (struct runctl *runp,
 		printf(_("  MIME decoding is %s (mimedecode %s).\n"),
 		       ctl->mimedecode ? _("enabled") : _("disabled"),
 		       ctl->mimedecode ? "on" : "off");
+		printf(_("  Idle after poll is %s (idle %s).\n"),
+		       ctl->idle ? _("enabled") : _("disabled"),
+		       ctl->idle ? "on" : "off");
 		printf(_("  Nonempty Status lines will be %s (dropstatus %s)\n"),
 		       ctl->dropstatus ? _("discarded") : _("kept"),
 		       ctl->dropstatus ? "on" : "off");
