@@ -35,6 +35,7 @@
 #define		A_PASSWORD	0	/* password or inline authentication */
 #define		A_KERBEROS_V4	1	/* preauthenticate w/ Kerberos V4 */
 #define		A_KERBEROS_V5	2	/* preauthenticate w/ Kerberos V5 */
+#define		A_SSH		3	/* preauthentication at session level */
 
 /*
  * Definitions for buffer sizes.  We get little help on setting maxima
@@ -259,6 +260,7 @@ struct query
     flag pass8bits;		/* if TRUE, ignore Content-Transfer-Encoding */
     flag dropstatus;		/* if TRUE, drop Status lines in mail */
     flag mimedecode;		/* if TRUE, decode MIME-armored messages */
+    flag idle;			/* if TRUE, idle after each poll */
     int	limit;			/* limit size of retrieved messages */
     int warnings;		/* size warning interval */
     int	fetchlimit;		/* max # msgs to get in single poll */
@@ -407,6 +409,9 @@ extern int stage;
 #define XMIT_REJECT	2	/* rejected; no match */
 #define XMIT_RCPTBAD	3	/* SMTP listener rejected the name */ 
 
+/* idle.c */
+int idle(int interval);
+
 /* sink.c: forwarding */
 int stuffline(struct query *, char *);
 int open_sink(struct query*, struct msgblk *, int*, int*);
