@@ -131,7 +131,7 @@ char *prompt;
 	signal(SIGINT, sig);
     }
     if (fi != stdin)
-	fclose(fi);
+	fclose(fi);	/* not checking should be safe, file mode was "r" */
 
     return(pbuf);
 #endif /* !(defined(HAVE_TCSETATTR) || ... */
@@ -190,7 +190,8 @@ static void restore_tty_state(void)
 static RETSIGTYPE sigint_handler(int signum)
 {
     restore_tty_state();
-    report(stderr, 1, 0, _("\nCaught signal... bailing out."));
+    report(stderr, _("\nCaught SIGINT... bailing out.\n"));
+    exit(1);
 }
 
 /* getpass.c ends here */
