@@ -64,6 +64,7 @@ extern char *strstr();	/* needed on sysV68 R3V7.1. */
 
 static int count, seen, recent, unseen, deletions, imap_version, preauth; 
 static int expunged, expunge_period;
+static flag do_idle, idling;
 static char capabilities[MSGBUFSIZE+1];
 
 int imap_ok(int sock, char *argbuf)
@@ -88,7 +89,21 @@ int imap_ok(int sock, char *argbuf)
 	if (strstr(buf, "* CAPABILITY"))
 	    strncpy(capabilities, buf + 12, sizeof(capabilities));
 	if (strstr(buf, "EXISTS"))
+	{
 	    count = atoi(buf+2);
+	    /*
+	     * Nasty kluge to handle RFC2177 IDLE.  If we know we're idling
+	     * we can't wait for the tag matching the IDLE; we have to tell the
+	     * server the IDLE is finished by shipping back a DONE when we
+	     * see an EXISTS.  Only after that will a tagged response be
+	     * shipped.  The idling flag also gets cleared on a timeout.
+	     */
+	    if (idling)
+	    {
+		gen_send(sock, "DONE");
+		idling = FALSE;
+	    }
+	}
 	if (strstr(buf, "RECENT"))
 	    recent = atoi(buf+2);
 	if (strstr(buf, "UNSEEN"))
@@ -858,6 +873,7 @@ int imap_getauth(int sock, struct query *ctl, char *greeting)
 
     /* probe to see if we're running IMAP4 and can use RFC822.PEEK */
     capabilities[0] = '\0';
+    preauth = FALSE;
     if ((ok = gen_transact(sock, "CAPABILITY")) == PS_SUCCESS)
     {
 	/* UW-IMAP server 10.173 notifies in all caps */
@@ -894,9 +910,25 @@ int imap_getauth(int sock, struct query *ctl, char *greeting)
     else
 	expunge_period = 1;
 
-    if (preauth)
+    /* 
+     * If either (a) we saw a PREAUTH token in the capability response, or
+     * (b) the user specified ssh preauthentication, then we're done.
+     */
+    if (preauth || ctl->server.preauthenticate == A_SSH)
 	return(PS_SUCCESS);
 
+    /* 
+     * Handle idling.  We depend on coming through here on startup
+     * and after each timeout (including timeouts during idles).
+     */
+    idling = FALSE;
+    if (strstr(capabilities, "IDLE") && ctl->idle)
+    {
+	do_idle = TRUE;
+	if (outlevel >= O_VERBOSE)
+	    report(stdout, "will idle after poll\n");
+    }
+
 #if OPIE_ENABLE
     if ((ctl->server.protocol == P_IMAP) && strstr(capabilities, "AUTH=X-OTP"))
     {
@@ -1046,14 +1078,15 @@ static int imap_getrange(int sock,
 	 */
 	ok = 0;
 	if (deletions && expunge_period != 1)
-	    internal_expunge(sock);
+	    ok = internal_expunge(sock);
 	count = -1;
-	if (ok || gen_transact(sock, "NOOP"))
+	idling = do_idle;
+	if (ok || gen_transact(sock, do_idle ? "IDLE" : "NOOP"))
 	{
 	    report(stderr, _("re-poll failed\n"));
 	    return(ok);
 	}
-	else if (count == -1)	/* no EXISTS response to NOOP */
+	else if (count == -1)	/* no EXISTS response to NOOP/IDLE */
 	{
 	    count = recent = 0;
 	    unseen = -1;
@@ -1061,10 +1094,9 @@ static int imap_getrange(int sock,
     }
     else
     {
-	if (!check_only)
-	    ok = gen_transact(sock, "SELECT %s", folder ? folder : "INBOX");
-	else
-	    ok = gen_transact(sock, "EXAMINE %s", folder ? folder : "INBOX");
+	ok = gen_transact(sock, 
+			  check_only ? "EXAMINE \"%s\"" : "SELECT \"%s\"",
+			  folder ? folder : "INBOX");
 	if (ok != 0)
 	{
 	    report(stderr, _("mailbox selection failed\n"));
@@ -1102,6 +1134,35 @@ static int imap_getsizes(int sock, int count, int *sizes)
      * Some servers (as in, PMDF5.1-9.1 under OpenVMS 6.1)
      * won't accept 1:1 as valid set syntax.  Some implementors
      * should be taken out and shot for excessive anality.
+     *
+     * Microsoft Exchange (brain-dead piece of crap that it is) 
+     * sometimes gets its knickers in a knot about bodiless messages.
+     * You may see responses like this:
+     *
+     *	fetchmail: IMAP> A0004 FETCH 1:9 RFC822.SIZE
+     *	fetchmail: IMAP< * 2 FETCH (RFC822.SIZE 1187)
+     *	fetchmail: IMAP< * 3 FETCH (RFC822.SIZE 3954)
+     *	fetchmail: IMAP< * 4 FETCH (RFC822.SIZE 1944)
+     *	fetchmail: IMAP< * 5 FETCH (RFC822.SIZE 2933)
+     *	fetchmail: IMAP< * 6 FETCH (RFC822.SIZE 1854)
+     *	fetchmail: IMAP< * 7 FETCH (RFC822.SIZE 34054)
+     *	fetchmail: IMAP< * 8 FETCH (RFC822.SIZE 5561)
+     *	fetchmail: IMAP< * 9 FETCH (RFC822.SIZE 1101)
+     *	fetchmail: IMAP< A0004 NO The requested item could not be found.
+     *
+     * This means message 1 has only headers.  For kicks and grins
+     * you can telnet in and look:
+     *	A003 FETCH 1 FULL
+     *	A003 NO The requested item could not be found.
+     *	A004 fetch 1 rfc822.header
+     *	A004 NO The requested item could not be found.
+     *	A006 FETCH 1 BODY
+     *	* 1 FETCH (BODY ("TEXT" "PLAIN" ("CHARSET" "US-ASCII") NIL NIL "7BIT" 35 3))
+     *	A006 OK FETCH completed.
+     *
+     * To get around this, we terminate the read loop on a NO and count
+     * on the fact that the sizes array has been preinitialized with a
+     * known-bad size value.
      */
     if (count == 1)
 	gen_send(sock, "FETCH 1 RFC822.SIZE", count);
@@ -1113,7 +1174,7 @@ static int imap_getsizes(int sock, int count, int *sizes)
 
 	if ((ok = gen_recv(sock, buf, sizeof(buf))))
 	    return(ok);
-	if (strstr(buf, "OK"))
+	else if (strstr(buf, "OK") || strstr(buf, "NO"))
 	    break;
 	else if (sscanf(buf, "* %d FETCH (RFC822.SIZE %d)", &num, &size) == 2)
 	    sizes[num - 1] = size;
