@@ -22,7 +22,11 @@ typedef unsigned char *POINTER;
 typedef unsigned short int UINT2;
 
 /* UINT4 defines a four byte word */
+#if SIZEOF_INT == 4
+typedef unsigned int UINT4;
+#else
 typedef unsigned long int UINT4;
+#endif
 
 /* PROTO_LIST is defined depending on how PROTOTYPES is defined above.
 If using PROTOTYPES, then PROTO_LIST returns the list, otherwise it
