@@ -51,7 +51,7 @@ static int smtp_open(struct query *ctl)
     /* maybe it's time to close the socket in order to force delivery */
     if (NUM_NONZERO(ctl->batchlimit) && (ctl->smtp_socket != -1) && ++batchcount == ctl->batchlimit)
     {
-	close(ctl->smtp_socket);
+	SockClose(ctl->smtp_socket);
 	ctl->smtp_socket = -1;
 	batchcount = 0;
     }
@@ -282,14 +282,19 @@ static int send_bouncemail(struct query *ctl, struct msgblk *msg,
 		|| SMTP_data(sock) != SM_OK)
 	return(FALSE);
 
+    /* our first duty is to keep the sacred foo counters turning... */
     sprintf(boundary, 
-	    "om-mani-padme-hum-%d-%d-%ld", 
+	    "foo-mani-padme-hum-%d-%d-%ld", 
 	    (int)getpid(), (int)getppid(), time((time_t *)NULL));
 
     ts = rfc822timestamp();
 
     if (outlevel >= O_VERBOSE)
 	report(stdout, "SMTP: (bounce-message body)\n");
+    else
+	/* this will usually go to sylog... */
+	report(stderr, "mail from %s bounced to %s\n",
+	       daemon_name, bounce_to);
 
     /* bouncemail headers */
     SockPrintf(sock, "Return-Path: <>\r\n");
@@ -372,7 +377,7 @@ static int send_bouncemail(struct query *ctl, struct msgblk *msg,
 
 static int handle_smtp_report(struct query *ctl, struct msgblk *msg)
 /* handle SMTP errors based on the content of SMTP_response */
-/* Mail is deleted from the server if this function returns PS_REFUSED. */
+/* return of PS_REFUSED deletes mail from the server; PS_TRANSIENT keeps it */
 {
     int smtperr = atoi(smtp_response);
     char *responses[1];
@@ -380,6 +385,11 @@ static int handle_smtp_report(struct query *ctl, struct msgblk *msg)
     xalloca(responses[0], char *, strlen(smtp_response)+1);
     strcpy(responses[0], smtp_response);
 
+    SMTP_rset(ctl->smtp_socket);    /* stay on the safe site */
+
+    if (outlevel >= O_DEBUG)
+	report(stdout, "Saved error is still %d\n", smtperr);
+
     /*
      * Note: send_bouncemail message strings are not made subject
      * to gettext translation because (a) they're going to be 
@@ -403,7 +413,6 @@ static int handle_smtp_report(struct query *ctl, struct msgblk *msg)
 	 * 554 = Postfix antispam response.
 	 *
 	 */
-	SMTP_rset(ctl->smtp_socket);    /* stay on the safe site */
 	send_bouncemail(ctl, msg, XMIT_ACCEPT,
 			"Our spam filter rejected this transaction.\r\n", 
 			1, responses);
@@ -422,32 +431,14 @@ static int handle_smtp_report(struct query *ctl, struct msgblk *msg)
 
     switch (smtperr)
     {
-    case 452: /* insufficient system storage */
-	/*
-	 * Temporary out-of-queue-space condition on the
-	 * ESMTP server.  Don't try to ship the message, 
-	 * and suppress deletion so it can be retried on
-	 * a future retrieval cycle. 
-	 *
-	 * Bouncemail *might* be appropriate here as a delay
-	 * notification (note; if we ever add this, we must make
-	 * sure the RFC1894 Action field is "delayed" rather thwn
-	 * "failed").  But it's not really necessary because
-	 * this is not an actual failure, we're very likely to be
-	 * able to recover on the next cycle.
-	 */
-	SMTP_rset(ctl->smtp_socket);    /* required by RFC1870 */
-	return(PS_TRANSIENT);
-
     case 552: /* message exceeds fixed maximum message size */
 	/*
 	 * Permanent no-go condition on the
 	 * ESMTP server.  Don't try to ship the message, 
 	 * and allow it to be deleted.
 	 */
-	SMTP_rset(ctl->smtp_socket);    /* required by RFC1870 */
 	send_bouncemail(ctl, msg, XMIT_ACCEPT,
-			"This message was too large.\r\n", 
+			"This message was too large (SMTP error 552).\r\n", 
 			1, responses);
 	return(run.bouncemail ? PS_REFUSED : PS_TRANSIENT);
   
@@ -459,19 +450,35 @@ static int handle_smtp_report(struct query *ctl, struct msgblk *msg)
 	 * (b) we wouldn't want spammers to get confirmation that
 	 * this address is live, anyway.
 	 */
-	SMTP_rset(ctl->smtp_socket);    /* stay on the safe side */
 	send_bouncemail(ctl, msg, XMIT_ACCEPT,
-			"Invalid address.\r\n", 
+			"Invalid address in MAIL FROM (SMTP error 553).\r\n", 
 			1, responses);
 	return(PS_REFUSED);
 
-    default:	/* bounce non-transient errors back to the sender */
-	SMTP_rset(ctl->smtp_socket);    /* stay on the safe side */
+    default:
+	/* bounce non-transient errors back to the sender */
 	if (smtperr >= 500 && smtperr <= 599)
 	    if (send_bouncemail(ctl, msg, XMIT_ACCEPT,
 				"General SMTP/ESMTP error.\r\n", 
 				1, responses))
 		return(run.bouncemail ? PS_REFUSED : PS_TRANSIENT);
+	/*
+	 * We're going to end up here on 4xx errors, like:
+	 *
+	 * 451: temporarily unable to identify sender (exim)
+	 * 452: temporary out-of-queue-space condition on the ESMTP server.
+	 *
+	 * These are temporary errors.  Don't try to ship the message,
+	 * and suppress deletion so it can be retried on a future
+	 * retrieval cycle.
+	 *
+	 * Bouncemail *might* be appropriate here as a delay
+	 * notification (note; if we ever add this, we must make
+	 * sure the RFC1894 Action field is "delayed" rather thwn
+	 * "failed").  But it's not really necessary because
+	 * these are not actual failures, we're very likely to be
+	 * able to recover on the next cycle.
+	 */
 	return(PS_TRANSIENT);
     }
 }
@@ -712,22 +719,38 @@ int open_sink(struct query *ctl, struct msgblk *msg,
 
 	/*
 	 * Try to get the SMTP listener to take the Return-Path
-	 * address as MAIL FROM .  If it won't, fall back on the
-	 * calling-user ID.  This won't affect replies, which use the
-	 * header From address anyway.
+	 * address as MAIL FROM.  If it won't, fall back on the
+	 * remotename and mailserver host.  This won't affect replies,
+	 * which use the header From address anyway; the MAIL FROM
+	 * address is a place for the SMTP listener to send
+	 * bouncemail.  The point is to guarantee a FQDN in the MAIL
+	 * FROM line -- some SMTP listeners, like smail, become
+	 * unhappy otherwise.
 	 *
 	 * RFC 1123 requires that the domain name part of the
 	 * MAIL FROM address be "canonicalized", that is a
-	 * FQDN or MX but not a CNAME.  We'll assume the From
+	 * FQDN or MX but not a CNAME.  We'll assume the Return-Path
 	 * header is already in this form here (it certainly
 	 * is if rewrite is on).  RFC 1123 is silent on whether
 	 * a nonexistent hostname part is considered canonical.
 	 *
 	 * This is a potential problem if the MTAs further upstream
 	 * didn't pass canonicalized From/Return-Path lines, *and* the
-	 * local SMTP listener insists on them.
+	 * local SMTP listener insists on them. 
 	 */
-	ap = (msg->return_path[0]) ? msg->return_path : user;
+	if (!msg->return_path[0])
+	{
+	    sprintf(addr, "%s@%s", ctl->remotename, ctl->server.truename);
+	    ap = addr;
+	}
+	else if (strchr(msg->return_path, '@'))
+	    ap = msg->return_path;
+	else		/* in case Return-Path existed but was local */
+	{
+	    sprintf(addr, "%s@%s", msg->return_path, ctl->server.truename);
+	    ap = addr;
+	}
+
 	if (SMTP_from(ctl->smtp_socket, ap, options) != SM_OK)
 	    return(handle_smtp_report(ctl, msg));
 
