@@ -121,7 +121,7 @@ int SockCheckOpen(int fd)
 int SockOpen(const char *host, const char *service, const char *options,
 	     const char *plugin)
 {
-    struct addrinfo *ai, req;
+    struct addrinfo *ai, *ai0, req;
     int i;
 #if NET_SECURITY
     void *request = NULL;
@@ -135,10 +135,10 @@ int SockOpen(const char *host, const char *service, const char *options,
     memset(&req, 0, sizeof(struct addrinfo));
     req.ai_socktype = SOCK_STREAM;
 
-    if (getaddrinfo(host, service, &req, &ai)) {
+    if (getaddrinfo(host, service, &req, &ai0)) {
 	report(stderr, _("fetchmail: getaddrinfo(%s.%s)\n"), host,service);
 	return -1;
-    };
+    }
 
 #if NET_SECURITY
     if (!options)
@@ -147,32 +147,36 @@ int SockOpen(const char *host, const char *service, const char *options,
 	if (net_security_strtorequest((char *)options, &request, &requestlen))
 	    goto ret;
 
-    i = inner_connect(ai, request, requestlen, NULL, NULL, "fetchmail", NULL);
+    i = inner_connect(ai0, request, requestlen, NULL, NULL, "fetchmail", NULL);
     if (request)
 	free(request);
 
  ret:
 #else /* NET_SECURITY */
 #ifdef HAVE_INNER_CONNECT
-    i = inner_connect(ai, NULL, 0, NULL, NULL, "fetchmail", NULL);
+    i = inner_connect(ai0, NULL, 0, NULL, NULL, "fetchmail", NULL);
+    if (i >= 0)
+	break;
 #else
-    i = socket(ai->ai_family, ai->ai_socktype, 0);
-    if (i < 0) {
-	freeaddrinfo(ai);
-	return -1;
-    }
-    if (connect(i, (struct sockaddr *) ai->ai_addr, ai->ai_addrlen) < 0) {
-	freeaddrinfo(ai);
-	close(i);
-	return -1;
+    i = -1;
+    for (ai = ai0; ai; ai = ai->ai_next) {
+	i = socket(ai->ai_family, ai->ai_socktype, 0);
+	if (i < 0)
+	    continue;
+	if (connect(i, (struct sockaddr *) ai->ai_addr, ai->ai_addrlen) < 0) {
+	    close(i);
+	    i = -1;
+	    continue;
+	}
+	break;
     }
 #endif
 #endif /* NET_SECURITY */
 
-    freeaddrinfo(ai);
+    freeaddrinfo(ai0);
 
     return i;
-};
+}
 #else /* INET6_ENABLE */
 #ifndef HAVE_INET_ATON
 #ifndef  INADDR_NONE
@@ -226,7 +230,7 @@ int SockOpen(const char *host, int clientPort, const char *options,
         if (connect(sock, (struct sockaddr *) &ad, sizeof(ad)) < 0)
         {
             int olderr = errno;
-            close(sock);
+            close(sock);	/* don't use SockClose, no traffic yet */
             h_errno = 0;
             errno = olderr;
             return -1;
@@ -272,14 +276,14 @@ int SockOpen(const char *host, int clientPort, const char *options,
 	    memcpy(&ad.sin_addr, *pptr, sizeof(struct in_addr));
 	    if (connect(sock, (struct sockaddr *) &ad, sizeof(ad)) == 0)
 		break; /* success */
-	    close(sock);
+	    close(sock);	/* don't use SockClose, no traffic yet */
 	    memset(&ad, 0, sizeof(ad));
 	    ad.sin_family = AF_INET;
 	}
 	if(*pptr == NULL)
 	{
 	    int olderr = errno;
-	    close(sock);
+	    close(sock);	/* don't use SockClose, no traffic yet */
 	    h_errno = 0;
 	    errno = olderr;
 	    return -1;
@@ -651,8 +655,9 @@ int SSLOpen(int sock, char *mycert, char *mykey, char *servercname )
 #endif
 
 int SockClose(int sock)
-/* close a socket (someday we may do other cleanup here) */
+/* close a socket gracefully */
 {
+    char ch;
 #ifdef	SSL_ENABLE
     SSL *ssl;
 
@@ -662,15 +667,31 @@ int SockClose(int sock)
         _ssl_context[sock] = NULL;
     }
 #endif
-    return(close(sock));
+
+    /* Half-close the connection first so the other end gets notified.
+     *
+     * This stops sends but allows receives (effectively, it sends a
+     * TCP <FIN>).
+     */
+    if (shutdown(sock, 1) == 0)
+	/* If there is any data still waiting in the queue, discard it.
+	 * Call recv() until either it returns 0 (meaning we received a FIN)
+	 * or any error occurs.  This makes sure all data sent by the other
+	 * side is acknowledged at the TCP level.
+	 */
+	if (recv(sock, &ch, 1, MSG_PEEK) > 0)
+	    while (read(sock, &ch, 1) > 0)
+		continue;
+
+    /* if there's an error closing at this point, not much we can do */
+    return(close(sock));	/* this is guarded */
 }
 
 #ifdef MAIN
 /*
  * Use the chargen service to test input buffering directly.
  * You may have to uncomment the `chargen' service description in your
- * inetd.conf (and then SIGHUP inetd) for this to work. 
- */
+ * inetd.conf (and then SIGHUP inetd) for this to work.  */
 main()
 {
     int	 	sock = SockOpen("localhost", 19, NULL);
