@@ -124,12 +124,26 @@ void initialize_saved_lists(struct query *hostlist, const char *idfile)
 	    /* first, skip leading spaces */
 	    user = buf + strspn(buf, " \t");
 
-	    /* First, we split the buf into a userhost part and an id part */
-	    if ((id = strchr(user, '<')) != NULL ) 	/* set pointer to id */
+	    /*
+	     * First, we split the buf into a userhost part and an id
+	     * part ... but id doesn't necessarily start with a '<',
+	     * espescially if the POP server returns an X-UIDL header
+	     * instead of a Message-ID, as GMX's (www.gmx.net) POP3
+	     * StreamProxy V1.0 does.
+	     */
+	    if ((id = strchr(user, ' ')) != NULL )
 	    {
 	        for (delimp1 = id; delimp1 >= user; delimp1--)
 		    if ((*delimp1 != ' ') && (*delimp1 != '\t'))
 			break;
+
+		/* 
+		 * It should be safe to assume that id starts after
+		 * the " " - after all, we're writing the " "
+		 * ourselves in write_saved_lists() :-)
+		 */
+		id = id + strspn(id, " ");
+
 		delimp1++; /* but what if there is only white space ?!? */
 	  	saveddelim1 = *delimp1;	/* save char after token */
 		*delimp1 = '\0';		/* delimit token with \0 */
@@ -397,24 +411,36 @@ void expunge_uids(struct query *ctl)
 void uid_end_query(struct query *ctl) 
 /* finish a query */
 {
-    /* old state of mailbox is now irrelevant */
-    free_str_list(&ctl->oldsaved);
-    free_str_list(&scratchlist);
-    ctl->oldsaved = ctl->newsaved;
-    ctl->newsaved = (struct idlist *) NULL;
-
     /* debugging code */
     if (ctl->server.uidl && outlevel >= O_DEBUG)
     {
 	struct idlist *idp;
 
 	report_build(stdout, "New UID list from %s:", ctl->server.pollname);
-	for (idp = ctl->oldsaved; idp; idp = idp->next)
+	for (idp = ctl->newsaved; idp; idp = idp->next)
 	    report_build(stdout, " %s = %d", idp->id, idp->val.status.mark);
 	if (!idp)
 	    report_build(stdout, " <empty>");
 	report_complete(stdout, "\n");
     }
+
+    /*
+     * Don't swap UID lists unless we've actually seen UIDLs.
+     * This is necessary in order to keep UIDL information
+     * from being heedlessly deleted later on.
+     */
+    if (ctl->newsaved)
+    {
+	/* old state of mailbox may now be irrelevant */
+	if (outlevel >= O_DEBUG)
+	    report(stdout, "swapping UID lists\n");
+	free_str_list(&ctl->oldsaved);
+	free_str_list(&scratchlist);
+	ctl->oldsaved = ctl->newsaved;
+	ctl->newsaved = (struct idlist *) NULL;
+    }
+    else if (outlevel >= O_DEBUG)
+	report(stdout, "not swapping UID lists, no UIDs seen this query\n");
 }
 
 void write_saved_lists(struct query *hostlist, const char *idfile)
@@ -440,6 +466,9 @@ void write_saved_lists(struct query *hostlist, const char *idfile)
 	unlink(idfile);
     }
     else
+    {
+	if (outlevel >= O_DEBUG)
+	    report(stdout, "Writing fetchids file.\n");
 	if ((tmpfp = fopen(idfile, "w")) != (FILE *)NULL) {
 	    for (ctl = hostlist; ctl; ctl = ctl->next) {
 		for (idp = ctl->oldsaved; idp; idp = idp->next)
@@ -452,6 +481,7 @@ void write_saved_lists(struct query *hostlist, const char *idfile)
 		fputs(idp->id, tmpfp);
 	    fclose(tmpfp);
 	}
+    }
 }
 #endif /* POP3_ENABLE */
 
